/* -*- Mode: c; indent-tabs-mode: nil; c-basic-offset: 8 -*-
 * vim:expandtab:shiftwidth=8:tabstop=8:
 *
 * Copyright (C) 2002  Andres Salomon <dilinger@mp3revolution.net>
 *
 *   This file is part of InterMezzo, http://www.inter-mezzo.org.
 *
 *   InterMezzo is free software; you can redistribute it and/or
 *   modify it under the terms of version 2 of the GNU General Public
 *   License as published by the Free Software Foundation.
 *
 *   InterMezzo is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with InterMezzo; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <string.h>
#include <glib.h>
#include <izolib.h>

/*
 * Parse $QUERY_STRING into a nice, handy GHashTable.
 */
static GHashTable *parse_query_str(void)
{
        GHashTable *table;
        const char *q;

        q = g_getenv("QUERY_STRING");
        table = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);

        while (q) {
                char *next, *ptr;

                next = strchr(q, '&');
                if (next)
                        *next++ = '\0';

                ptr = strchr(q, '=');
                if (ptr) {
                        *ptr++ = '\0';
                        g_hash_table_insert(table, g_strdup(q), g_strdup(ptr));
                }

                q = next;
        }

        return table;
}

const char *cgi_getenv(const char *var)
{
        static GHashTable *cgi_var_table = NULL;

        if (cgi_var_table == NULL)
                cgi_var_table = parse_query_str();

        return (const char *) g_hash_table_lookup(cgi_var_table, var);
}

int cgi_getenv_u64(const char *var, __u64 *val)
{
        const char *str = cgi_getenv(var);
        /* should this return a status, and pass a pointer for the data? */
        if (str == NULL) {
                fprintf (stderr, "Missing '%s'\n", var);
                return -1;
        }

        *val = strtoul(str, NULL, 0);
        return 0;
}
