/* -*- mode: c; c-basic-offset: 8; indent-tabs-mode: nil; -*-
 * vim:expandtab:shiftwidth=8:tabstop=8:
 *
 *  Copyright (C) 2002 Cluster File Systems, Inc.
 *                     Author: Gordon Matzigkeit <gord@fig.org>
 *
 *   This file is part of InterMezzo, http://www.inter-mezzo.org.
 *
 *   InterMezzo is free software; you can redistribute it and/or
 *   modify it under the terms of version 2 of the GNU General Public
 *   License as published by the Free Software Foundation.
 *
 *   InterMezzo is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with InterMezzo; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Build a GList of KML records.
 *
 */

#include <sys/stat.h>
#include <unistd.h>
#include <sys/mman.h>
#include <fcntl.h>

#include <string.h>
#include <error.h>
#include <errno.h>

#include <linux/intermezzo_fs.h>

#include <glib.h>
#include <izolib.h>

GList *kml_listify(char *buf, int len, int first_recno)
{
        GList *l = NULL;
        char *ptr, *end;
        struct kml_rec *rec = NULL;
        int prior;

        ptr = buf;
        end = buf + len;
        prior = -1;
        while ( ptr < end ) {
                int *i = (int *)ptr;
                if (*i)
                        break;
                ptr = (char *)ptr + sizeof(*i);
        }
        while ( ptr < end ) { 
                int err;
                if (!rec)
                        rec = g_new(struct kml_rec, 1);
                err = kml_unpack(rec, &ptr, end);
                if ( err ) {
                        error(0, -err, "unpacking: %d", err);
                        if (prior == -1)
                                g_error("failure to unpack first record\n");
                        else
                                g_error("failure to unpack record after %d!\n",
                                        prior);
                }
                prior = rec->suffix->recno;
                if (prior >= first_recno) {
                        l = g_list_append(l, rec);
                        rec = NULL;
                }
        }
        return l;
}

/* Load in several KML entries as a list. */
GList *kml_listify_file(char *fname, int first_recno)
{
        char *buf;
        int fd;
        struct stat st;

        fd = open(fname, O_RDONLY);
        if ( fd == -1)
                error(1, errno, "open %s fails", fname);

        if (fstat(fd, &st))
                error(1, errno, "stat %s fails", fname);

        if (! S_ISREG(st.st_mode))
                error(1, errno, "KML %s is not a file", fname);

        if (st.st_size < 1)
                error(1, 0, "KML %s is empty", fname);

        buf = mmap(NULL, st.st_size, PROT_READ|PROT_WRITE, MAP_PRIVATE, fd, 0);
        if ( !buf )
                error(1, errno, "mmap %s fails", fname);
        close(fd);

        return kml_listify(buf, st.st_size, first_recno);
}
