/* -*- Mode: c; indent-tabs-mode: nil; c-basic-offset: 8 -*-
 * vim:expandtab:shiftwidth=8:tabstop=8:
 *
 * Copyright (C) 2001, 2002 Cluster File Systems, Inc.
 * Copyright (C) 2001 Tacit Networks, Inc. <phil@off.net>
 *
 *   This file is part of InterMezzo, http://www.inter-mezzo.org.
 *
 *   InterMezzo is free software; you can redistribute it and/or
 *   modify it under the terms of version 2 of the GNU General Public
 *   License as published by the Free Software Foundation.
 *
 *   InterMezzo is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with InterMezzo; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <errno.h>

#include <glib.h>
#include <izolib.h>
#include <linux/intermezzo_lib.h>
#include <uuid/uuid.h>

int main(int argc, char **argv)
{
	char message [] =
		//"HTTP/1.1 %d\r\n"
		"Connection: Keep-Alive\r\n"
		"Content-Type: text/plain\r\n"
		"Content-Length: %d\r\n\r\n"
		"%d";
        const char *fsetname, *uuidstr;
        char *docroot;
        int fd, rc = 0;
        uuid_t uuid;

        docroot = getenv("DOCUMENT_ROOT");
        if (docroot == NULL) {
                fprintf(stderr, "missing docroot\n");
                rc = -EINVAL;
                goto error;
        }

        /* query should look something like "fset=testvol&uuid=blahblah" */
        fsetname = cgi_getenv("fset");
        uuidstr = cgi_getenv("uuid");
        if (fsetname == NULL || uuidstr == NULL) {
                fprintf(stderr, "missing fset or uuid in query string\n");
                rc = -EINVAL;
                goto error;
        }

        if (uuid_parse((char *)uuidstr, uuid) != 0) {
                fprintf(stderr, "Error: UUID \"%s\" is not valid.\n", uuidstr);
                rc = -EINVAL;
                goto error;
        }

        fd = fset_cgi_open(docroot, fsetname);
        if (fd < 0) {
                rc = -errno;
                goto error;
        }

        /* Tell the kernel (and, indirectly, intersync) that it no longer has
         * the permit. */
        rc = izo_ioc_revoke_permit(fd, uuid);
        if (rc < 0) {
                fprintf(stderr, "izo_ioc_revoke_permit failed: %s\n",
                        g_strerror(-rc));
                goto error;
        }

 error:
        printf(message, 1, rc);
	return 0;
}

