/* -*- Mode: c; indent-tabs-mode: nil; c-basic-offset: 8 -*-
 * vim:expandtab:shiftwidth=8:tabstop=8:
 *
 * Copyright (C) 2001, 2002 Cluster File Systems, Inc.
 *   Author: Phil Schwan <phil@clusterfs.com>
 *
 *   This file is part of InterMezzo, http://www.inter-mezzo.org.
 *
 *   InterMezzo is free software; you can redistribute it and/or
 *   modify it under the terms of version 2 of the GNU General Public
 *   License as published by the Free Software Foundation.
 *
 *   InterMezzo is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with InterMezzo; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <errno.h>

#include <glib.h>
#include <izolib.h>
#include <linux/intermezzo_lib.h>

int main(int argc, char **argv)
{
        char message [] =
                //"HTTP/1.1 %d\r\n"
                "Connection: Keep-Alive\r\n"
                "Content-Type: text/plain\r\n"
                "Content-Length: %d\r\n\r\n"
                "%d";
        const char *fsetname, *kmlstr, *uuidstr;
        char *docroot, *filename;
        int fd, rc = -1;
        __u64 kmlsize;
        uuid_t uuid;

        docroot = getenv("DOCUMENT_ROOT");
        if (docroot == NULL) {
                fprintf(stderr, "missing docroot\n");
                goto error;
        }

        /* query should look something like "fset=testvol&uuid=blahblah" */
        fsetname = cgi_getenv("fset");
        uuidstr = cgi_getenv("uuid");
        kmlstr = cgi_getenv("kmlsize");

        if (fsetname == NULL || kmlstr == NULL || uuidstr == NULL) {
                fprintf(stderr,
                        "missing fset, kmlsize, or uuid in query string\n");
                goto error;
        }

        kmlsize = strtoul(kmlstr, NULL, 0);
        sscanf(kmlstr, "%llu", &kmlsize);
        if (uuid_parse((char *)uuidstr, uuid) != 0) {
                fprintf(stderr, "Error: UUID \"%s\" is not valid.\n", uuidstr);
                goto error;
        }

        filename = g_strdup_printf("%s/cache/.intermezzo/%s/ROOT", docroot,
                                   fsetname);
        fd = open(filename, O_RDONLY);
        if (fd < 0) {
                fprintf(stderr, "open(%s, O_RDONLY) failed: %s\n", filename,
                        g_strerror(errno));
                g_free(filename);
                goto error;
        }
        g_free(filename);

        rc = izo_ioc_set_kmlsize(fd, kmlsize, uuid);
        if (rc < 0) {
                fprintf(stderr, "izo_ioc_set_kmlsize failed: %s\n",
                        g_strerror(-rc));
                goto error;
        }

        rc = 0;

 error:
        printf(message, 1, (rc == 0));
        return 0;
}

