/* -*- Mode: c; indent-tabs-mode: nil; c-basic-offset: 8 -*-
 * vim:expandtab:shiftwidth=8:tabstop=8:
 *
 * Copyright (C) 2001, 2002 Cluster File Systems, Inc.
 *   Author: Phil Schwan <phil@clusterfs.com>
 *   Author: Peter J. Braam <braam@clusterfs.com>
 *
 *   This file is part of InterMezzo, http://www.inter-mezzo.org.
 *
 *   InterMezzo is free software; you can redistribute it and/or
 *   modify it under the terms of version 2 of the GNU General Public
 *   License as published by the Free Software Foundation.
 *
 *   InterMezzo is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with InterMezzo; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <errno.h>
#include <unistd.h>

#include <izolib.h>
#include <linux/intermezzo_lib.h>

int main(int argc, char **argv)
{
        struct is_config *cfg;
        char message [] =
                //"HTTP/1.1 %d\r\n"
                "Connection: Keep-Alive\r\n"
                "Content-Type: text/plain\r\n"
                "Content-Length: %d\r\n\r\n"
                "%s";
        const char *portstr, *uuidstr, *flagstr;
        char *filename, *docroot, *ipaddr;
        __u64 port = 0, ip;
        int fd, rc = 0, do_reverse = 0;
        uuid_t uuid;
        int client_flag;

        ipaddr = getenv("REMOTE_ADDR");
        docroot = getenv("DOCUMENT_ROOT");
        if (docroot == NULL) {
                fprintf(stderr, "missing docroot\n");
                goto error;
        }

        /* query should look something like 
         * "client_port=1873481&uuid=UUID_STRING&client_flag=[0|1]" 
         */
        portstr = cgi_getenv("client_port");
        if (portstr) {
                sscanf(portstr, "%llu", &port);
                if (port > 0)
                        do_reverse = 1;
        }
        uuidstr = cgi_getenv("uuid");
        if (!uuidstr || uuid_parse((char *)uuidstr, uuid) != 0) {
                fprintf(stderr, "Error: UUID \"%s\" is not valid.\n", uuidstr);
                rc = -EINVAL;
                goto error;
        }
        flagstr = cgi_getenv("client_flag");
        if (flagstr) 
                client_flag = flagstr[0] - '0';
        else
                client_flag = 0;

        if (ipaddr && izo_resolve(ipaddr, &ip)) {
                fprintf(stderr, "resolve failure %s", ipaddr); 
                /* so what do we do here ? */
        }

        /* ugh, can't get away from this... */
        if (client_flag) {
                fd = cache_cgi_open(docroot);
                if (fd < 0) {
                        rc = -errno;
                        goto error;
                }
                rc = izo_ioc_connect(fd, ip, port, uuid, client_flag);
                if (rc < 0) {
                        fprintf(stderr, "izo_ioc_connect failed: %s\n",
                                g_strerror(-rc));
                        goto error;
                }
                close(fd);
        }

 error:
        memset(&cfg, 0, sizeof(cfg));
        if (rc) {
                printf(message, 0, "");
                return 0;
        }

        filename = g_strdup_printf("%s/cache", docroot);
        cfg = read_local_config(filename);
        if (cfg == NULL || cfg->cfg_uuid == NULL) {
                printf(message, 0, "");
                g_free(filename);
                return 0;
        }
        g_free(filename);

        printf(message, strlen(cfg->cfg_uuid), cfg->cfg_uuid);
        g_free(cfg);

        return 0;
}

