/* -*- mode: c; c-basic-offset: 8; indent-tabs-mode: nil; -*-
 * vim:expandtab:shiftwidth=8:tabstop=8:
 */

#include <stdio.h>
#include <errno.h>
#include <asm/types.h>
#include <fcntl.h>

#include <uuid/uuid.h>
#include <linux/intermezzo_fs.h>

int main(int argc, char **argv)
{
        int fd;
        char *dev, *uuidstr, *buf;
        int rc;
        struct izo_ioctl_data ioc_data;
        struct izo_rcvd_rec rec;

        if (argc != 3) {
                printf("Usage: %s <directory> <uuid>\n", argv[0]);
                exit(1);
        }

        dev = argv[1];
        uuidstr = argv[2];

        memset(&ioc_data, 0, sizeof(ioc_data));

        if (uuid_parse(uuidstr, ioc_data.ioc_uuid) != 0) {
                printf("Error: UUID \"%s\" is not valid.\n", uuidstr);
                exit(1);
        }

        fd = open(dev, O_RDONLY);
        if (fd < 0) {
                printf("Error opening %s: %s\n", dev, strerror(errno));
                exit(errno);
        }

        buf = NULL;
        if (izo_ioctl_pack(&ioc_data, &buf, 0) != 0) {
                printf("izo_ioctl_pack failed\n");
                exit(1);
        }

        rc = ioctl(fd, IZO_IOC_GET_RCVD, buf);
        if (rc) {
                printf("Error in IZO_IOC_GET_RCVD ioctl: %s\n", strerror(errno));
                exit(rc);
        }

        memcpy(&rec, buf, sizeof(rec));
        uuid_unparse(rec.lr_uuid, uuidstr);
        printf("uuid                  : %s\n", uuidstr);
        printf("last remote recno     : %Lu\n", rec.lr_remote_recno);
        printf("last remote offset    : %Lu\n", rec.lr_remote_offset);
        printf("last local recno      : %Lu\n", rec.lr_local_recno);
        printf("last local offset     : %Lu\n", rec.lr_local_offset);
        printf("last backfetched ctime: %Lu\n", rec.lr_last_ctime);

        return 0;
}
