/* -*- Mode: c; indent-tabs-mode: nil; c-basic-offset: 8 -*-
 * vim:expandtab:shiftwidth=8:tabstop=8:
 *
 * Copyright (C) 2001, 2002 Cluster File Systems, Inc.
 *
 *   This file is part of InterMezzo, http://www.inter-mezzo.org.
 *
 *   InterMezzo is free software; you can redistribute it and/or
 *   modify it under the terms of version 2 of the GNU General Public
 *   License as published by the Free Software Foundation.
 *
 *   InterMezzo is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with InterMezzo; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#define printk printf
#include <unistd.h>
#include <sys/un.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <errno.h>
#include <string.h>

#define __KERNEL__
# include <linux/list.h>
#undef __KERNEL__

#include "parser.h"
#include <glib.h>
#include <stdio.h>
#include <uuid/uuid.h>
#include <izolib.h>
#include <linux/intermezzo_lib.h>

#include <pwd.h>
#include <linux/intermezzo_fs.h>
#include <linux/intermezzo_idl.h>

static int ic_clear_all_fsets(int argc, char **argv)
{
        int fd, rc;

        if (argc != 2) { 
                printf("Usage: %s mountpoint\n", argv[0]);
                return -1;
        }

        fd = open(argv[1], O_RDONLY); 
        if ( fd < 0 ) { 
                printf("Error opening %s: %s\n", argv[1], strerror(errno));
                return errno;
        }

        rc = izo_ioc_clear_all_fsets(fd);
        close(fd);
        return rc;
}

static int ic_get_flags(int argc, char **argv)
{
        int fd, rc;
        int file_flags, and_flags, or_flags;

        if (argc != 2) { 
                printf("Usage: %s file\n", argv[0]);
                return -1;
        }

        fd = open(argv[1], O_RDONLY); 
        if ( fd < 0 ) { 
                printf("Error opening %s: %s\n", argv[1], strerror(errno));
                return -1;
        }

        file_flags = 0;
        and_flags = ~0; /* becomes fset flags */
        or_flags = 0; /* becomes cache flags */
        rc = izo_ioc_get_flags(fd, &file_flags, &and_flags, &or_flags);

        if (rc < 0)
                printf("No flags.\n");
        else
                printf("File flags %x ; Fset flags %x ; Cache flags %x\n",
                       file_flags, and_flags, or_flags);

        close(fd);
        return rc;
}

static int ic_steal_permit(int argc, char **argv)
{
        int fd, rc, and, or;
        if (argc != 3) { 
                printf("Usage: %s mountpoint 0|1\n", argv[0]);
                return -1;
        }

        fd = open(argv[1], O_RDONLY); 
        if ( fd < 0 ) { 
                printf("Error opening %s: %s\n", argv[1], strerror(errno));
                return -1;
        }

        if (!strcmp(argv[2], "1")) {
                and = ~0;
                or = FSET_STEAL_PERMIT;
        } else if (!strcmp(argv[2], "0")) {
                and = ~FSET_STEAL_PERMIT;
                or = 0;
        } else {
                printf("Argument must be `0' or `1', not `%s'\n", argv[2]);
                return -1;
        }

        rc = izo_ioc_mark_fset(fd, and, or);
        close(fd);

        if (rc < 0)
                printf("mark_fset failed: %s\n", g_strerror(errno));

        return rc;
}


static int ic_set_ioc_uid(int argc, char **argv)
{
        char *dev;
        int rc, fd;
        int uid;
        struct passwd *passwd;

        if (argc != 3) {
                printf("Usage: %s <directory> <uid | username>\n", argv[0]);
                return -1;
        }

        dev = argv[1];

        passwd = getpwnam(argv[2]);
        if (passwd == NULL)
                uid = atoi(argv[2]);
        else
                uid = passwd->pw_uid;

        fd = open(dev, O_RDONLY);
        if (fd < 0) {
                printf("Error opening %s: %s\n", dev, strerror(errno));
                return -1;
        }

        rc = izo_ioc_set_uid(fd, uid); 

        if (close(fd)) { 
                g_error("error closing \n"); 
        }

        return rc;
}

static int ic_set_fset(int argc, char **argv)
{
        int fd, rc;
        

        if (argc < 3) { 
                printf("Usage: %s mountpoint fsetname [options]\n", argv[0]);
                return -1;
        }

        if (argc > 3) {
                printf("Options not supported yet.\n");
                return -1;
        }

        fd = open(argv[1], O_RDONLY); 
        if ( fd < 0 ) { 
                printf("Error opening %s: %s\n", argv[1], strerror(errno));
                return -1;
        }

        rc = izo_ioc_set_fset(fd, argv[2], 0);
        close(fd);

        if (rc < 0)
                printf("set_fset failed: %s\n", g_strerror(errno));

        return rc;
}

static int ic_uuid(int argc, char **argv)
{
        char *filename;

        if (argc < 1 || argc > 2) {
                g_warning("Usage: %s [filename]\n", argv[0]);
                return -1;
        }

        if (argc == 2) { 
                filename = argv[1];
        } else { 
                filename = "sysid"; 
        }
        /*config_create_sysid(NULL, filename); */                
        return 0 ;
}

static int ic_connect(int argc, char **argv)
{
        __u64 ip, port;
        int rc, fd;
        char *mountpoint, *hostname, *uuidstr;
        uuid_t uuid;
        int client_flag = 1;

        if (argc < 4 || argc > 5) {
                g_warning("Usage: %s <mountpoint> <hostname> <uuid> [port]\n", 
                          argv[0]);
                return -1;
        }
        if (argc == 4) {
                port = HTON__u64(370);
        } else {
                port = HTON__u64(atoi(argv[4]));
        }
        mountpoint = argv[1];
        hostname = argv[2];
        uuidstr = argv[3];

        if (uuid_parse((char *)uuidstr, uuid) != 0) {
                fprintf(stderr, "Error: UUID \"%s\" is not valid.\n", 
                        uuidstr);
                return -EINVAL;

        }

        fd = open(mountpoint, O_RDONLY);
        if (fd < 0) {
                g_warning("open(%s) failed: %s\n", mountpoint,
                          g_strerror(errno));
                return -1;
        }

        rc = izo_resolve(hostname, &ip);
        if (rc < 0) {
                g_warning("izo_resolve failed: %s\n", g_strerror(rc));
                return -1;
        }

        izo_ioc_connect(fd, ip, port, uuid, client_flag);
        close(fd);
        return 0;
}

static int ic_sync_remote(int argc, char **argv)
{
        int fd;
        __u64 kmlsize;
        char *mountpoint;

        if (argc != 2) {
                g_warning("Usage: %s <mountpoint>\n", argv[0]);
                return -1;
        }
        mountpoint = argv[1];

        fd = open(mountpoint, O_RDONLY);
        if (fd < 0) {
                g_warning("open(%s) failed: %s\n", mountpoint,
                          g_strerror(errno));
                return -1;
        }

        izo_ioc_get_kml_size(fd, &kmlsize);
        izo_ioc_go_fetch_kml(fd, 0, kmlsize);
        close(fd);
        return 0;
}

static int ic_make_branch(int argc, char **argv)
{
        int fd;
        char *mountpoint, *tagname, *branchname;

        if (argc != 4) {
                g_warning("Usage: %s <mountpoint> <tagname> <branchname>\n", argv[0]);
                return -1;
        }
        mountpoint = argv[1];
        tagname = argv[2];
        branchname = argv[3];

        fd = open(mountpoint, O_RDONLY);
        if (fd < 0) {
                g_warning("open(%s) failed: %s\n", mountpoint,
                          g_strerror(errno));
                return -1;
        }

        izo_ioc_client_make_branch(fd, tagname, branchname);
        close(fd);
        return 0;
}

static int ic_branch_undo(int argc, char **argv)
{
        int fd;
        char *mountpoint, *branchname;

        if (argc != 3) {
                g_warning("Usage: %s <mountpoint> <branchname>\n", argv[0]);
                return -1;
        }
        mountpoint = argv[1];
        branchname = argv[2];

        fd = open(mountpoint, O_RDONLY);
        if (fd < 0) {
                g_warning("open(%s) failed: %s\n", mountpoint,
                          g_strerror(errno));
                return -1;
        }

        izo_ioc_branch_undo(fd, branchname);
        close(fd);
        return 0;
}

static int ic_branch_redo(int argc, char **argv)
{
        int fd;
        char *mountpoint, *branchname;

        if (argc != 3) {
                g_warning("Usage: %s <mountpoint> <branchname>\n", argv[0]);
                return -1;
        }
        mountpoint = argv[1];
        branchname = argv[2];

        fd = open(mountpoint, O_RDONLY);
        if (fd < 0) {
                g_warning("open(%s) failed: %s\n", mountpoint,
                          g_strerror(errno));
                return -1;
        }

        izo_ioc_branch_redo(fd, branchname);
        close(fd);
        return 0;
}

command_t list[] = {
        {"connect", ic_connect, 0, "initiate reverse connection (args: directory, host, port)"},
        {"sync_remote", ic_sync_remote, 0, "instruct the remote system to fetch kml (args: directory)"},
        {"uuid", ic_uuid, 0, "generate a sysid file with a uuid (arg: filename)"},
        {"clear_all_fsets", ic_clear_all_fsets, 0, "clear all fsets in a given cache (arg: cache mountpoint)"},
        {"get_flags", ic_get_flags, 0, "get all flags (arg: filename)"},
        {"set_fset_root", ic_set_fset, 0, "mark a directory as fileset root (args: directory fileset [options]"},
        {"set_ioc_uid", ic_set_ioc_uid, 0, "set the user allowed to add KML (args: directory uid|username)"},
        {"steal_permit", ic_steal_permit, 0, "modify the fileset steal_permit flag (args: directory 0|1)"},
        {"make_branch", ic_make_branch, 0, "add a branch (args: mtpt tag branch)"},
        {"branch_undo", ic_branch_undo, 0, "remove a branch (args: mtpt branch)"},
        {"branch_redo", ic_branch_redo, 0, "roll forward onto a branch (args: mtpt branch)"},
        {"help", Parser_help, 0, "help"},
        {"exit", Parser_quit, 0, "quit"},
        {"quit", Parser_quit, 0, "quit"},
        { 0, 0, 0, NULL }
};

int main(int argc, char **argv)
{

        Parser_init("incontrol > ", list);

        if (argc > 1) { 
                return Parser_execarg(argc - 1, &argv[1], list);
        }

        Parser_commands();

        return 0;
}
