#include <sys/vfs.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/param.h>
#include <mntent.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>
#include <unistd.h>
#include <assert.h>
#include <pthread.h>
#include <libgen.h>
#include <linux/ext2_fs.h>
#include <ext2fs/ext2fs.h>
#include <fcntl.h>

int main(int argc, char **argv)
{
        int err; 
        int flags = 0;
        ext2_filsys fs;
        ext2_ino_t e2ino;
        struct ext2_inode e2inode;
        char device[128];
        struct stat buf;
        char ifilename[4096];
        char relname[4096];
        char *ptr;
        char *pdirb;
        char *dirb;
        char *dir;
        char *base;
        unsigned long ino;
        unsigned long generation;
        dev_t the_dev;
        int fd;

        if (argc != 2) { 
                fprintf(stderr, "usage %s filename\n", argv[0]);
                exit(1);
        }
        
        if ( stat(argv[1], &buf) ) { 
                fprintf(stderr, "cannot open %s\n", argv[1]);
                exit(1);
        }
        ino = buf.st_ino;
        generation = 0;

        the_dev = buf.st_dev;
        /* try devfs names first */
        if ( major(the_dev) == 3) { 
                snprintf(device, sizeof(device), "/dev/hda/%d",
                         minor(the_dev));
                if (stat(device, &buf))
                        snprintf(device, sizeof(device), "/dev/hda%d", 
                                 minor(the_dev));
        } else if ( major(the_dev) == 7) { 
                snprintf(device, sizeof(device), "/dev/loop/%d", 
                         minor(the_dev));
                if (stat(device, &buf))
                        snprintf(device, sizeof(device), "/dev/loop%d", 
                                 minor(the_dev));
        }
        
        ptr = relname + sizeof(relname) - 1;
        memset(relname, 0, sizeof(relname));
        dirb = strdup(argv[1]);
        pdirb = strdup(argv[1]);
        while ( 1 ) { 
                char *dirc = strdup(dirb);
                char *basec = strdup(dirb);
                free(dirb);

                dir = dirname(dirc);
                dirb = strdup(dir);
                if ( stat(dir, &buf) ) { 
                        fprintf(stderr, "error statting %s\n", dir);
                        exit(2);
                }
                if (buf.st_dev != the_dev) {
                        free(dirc);
                        break;
                }
                free(pdirb);
                pdirb = strdup(dir);
                free(dirc);

                base = basename(basec);
                ptr -= strlen(base) + 1;
                memcpy(ptr, "/", 1);
                memcpy(ptr+ 1, base, strlen(base));

                free(basec);
        }
        
        if (major(the_dev) == 3 || major(the_dev) == 7) { 
                /* this guy will abort if it fails */
                err = ext2fs_open(device, flags, 0, 0, unix_io_manager, &fs);
                if (err) { 
                        fprintf(stderr, "error %d in ext2fs_open, device %s\n", err, device);
                        exit(1);
                }
                
                err = ext2fs_namei(fs, 2, 2, ptr, &e2ino);
                if (err) { 
                        fprintf(stderr, "error in lookup %d\n", err);
                        ext2fs_close(fs);
                        exit(1);
                }
                
                err= ext2fs_read_inode (fs, e2ino, &e2inode);
                if (err) { 
                        fprintf(stderr, "error in lookup %d\n", err);
                        ext2fs_close(fs);
                        exit(1);
                }
                ino = e2ino;
                generation = e2inode.i_generation;
        }

        snprintf(ifilename, sizeof(ifilename), "%s/..iopen../...ino:%lx:%lx", pdirb, 
                 ino, generation);

        fd = open(ifilename, O_RDWR);
        if (fd < 0) {
                fprintf(stderr, "Error i-opening %s (as %s) %s\n", 
                        ptr, ifilename, strerror(errno));
        } else { 
                fprintf(stderr, "Success i-opening %s (as %s)\n", 
                        ptr, ifilename);
        }
        return 0;
}
        
