/* -*- mode: c; c-basic-offset: 8; indent-tabs-mode: nil; -*-
 * vim:expandtab:shiftwidth=8:tabstop=8:
 *
 *  Copyright (C) 2001 Cluster File Systems, Inc. <braam@clusterfs.com>
 *
 *   This file is part of InterMezzo, http://www.inter-mezzo.org.
 *
 *   InterMezzo is free software; you can redistribute it and/or
 *   modify it under the terms of version 2 of the GNU General Public
 *   License as published by the Free Software Foundation.
 *
 *   InterMezzo is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with InterMezzo; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Printing of KML records
 *
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <linux/intermezzo_fs.h>
#include <glib.h>

int main(int argc, char **argv)
{
        char *buf, *end, *str, *ptr, *filename;
        int fd, recno, brief = 0;
	struct stat st;
	struct kml_rec rec;

	if ( argc < 2 || argc > 4 ) {
		printf("usage %s [--brief] kmlfile [recno]\n", argv[0]);
		exit(1);
	}

        if (strcmp(argv[1], "--brief") != 0) {
                filename = argv[1];
        } else {
                filename = argv[2];
                brief = 1;
        }

        if (argc > 2 + brief)
                recno = atoi(argv[2 + brief]);
	else
		recno = -1;

        fd = open(filename, O_RDONLY);
        if (fd == -1) {
		fprintf(stderr, "Unable to open %s: %s\n", filename,
                        strerror(errno));
		exit(1);
	}

	if (fstat(fd, &st)) { 
		perror("stat fails");
		exit(1);
	}

	if (st.st_size < 1) {
		fprintf(stderr, "KML is empty\n");
		exit(1);
	}

	buf = mmap(NULL, st.st_size, PROT_READ, MAP_SHARED, fd, 0);
	if ( !buf ) { 
		perror("mmap fails");
		exit(1);
	}

	ptr = buf;
	end = buf + st.st_size;
	while ( ptr < end ) { 
		int err;
		if (recno == -1)
			printf(" *****  ptr %p (buf %p, end %p) offset=%d **\n",
			       ptr, buf, end, ptr - buf);
		err = kml_unpack(&rec, &ptr, end);
		if (recno != -1 && rec.suffix->recno != recno)
			continue;
		if ( err ) {
			printf("unpacking: %d, %s", err, strerror(-err));
			exit(1);
		}

		if (recno == -1)
			printf(" *****  ptr %p (buf %p, end %p) offset=%d **\n",
			       ptr, buf, end, ptr - buf);

                str = kml_print_rec(&rec, brief);
                printf("%s\n", str);
                free(str);

		if (recno != -1 && rec.suffix->recno == recno)
			break;
	}

	exit(0);
}
